/*
 * jQuery Idle Timeout 1.1
 * Copyright (c) 2011 Eric Hynds
 *
 * http://www.erichynds.com/jquery/a-new-and-improved-jquery-idle-timeout-plugin/
 *
 * Depends:
 *  - jQuery 1.4.2+
 *  - jQuery Idle Timer (by Paul Irish, http://paulirish.com/2009/jquery-idletimer-plugin/)
 *
 * Dual licensed under the MIT and GPL licenses:
 *   http://www.opensource.org/licenses/mit-license.php
 *   http://www.gnu.org/licenses/gpl.html
 */
(function($) {
    $.idleTimeout = function(element, resume, options) {

        // overwrite $.idleTimeout.options with the results of $.extend.
        // Allows you to write and a callback then call it from from a jQuery UI button or something
        options = $.idleTimeout.options = $.extend({}, $.idleTimeout.options, options);

        var IdleTimeout = {
            init: function() {
                var self = this;

                this.warning = $(element);
                this.resume = $(resume);
                this.countdownOpen = false;
                this.counter = -1;
                this.lastActivity = +new Date;

                // once the user becomes idle
                $(document).bind("idle.idleTimer", function() {
                    // if a countdown isn't already running
                    if (!self.countdownOpen) {
                        self._stopTimer();
                        self.countdownOpen = true;
                        self.counter = -1;
                        self._keepAlive();
                    }
                });

                $(document).bind("reset.idleTimer", function() {
                    if (!self.countdownOpen) {
                        self.lastActivity = +new Date;
                    }
                });

                // bind continue link
                this.resume.bind("click", function(e) {
                    e.preventDefault();

                    window.clearInterval(self.countdown); // stop the countdown
                    self.countdownOpen = false; // stop countdown
                    self.counter = -1;
                    self.lastActivity = +new Date;
                    options.onResume.call(self.warning); // call the resume callback

                    // restart start the idle timer
                    $.idleTimer('destroy');
                    $.idleTimer(options.idleAfter * 1000);

                    self._keepAlive();  // restart keepAlive messages
                });

                // start the idle timer
                $.idleTimer(options.idleAfter * 1000);

                this._keepAlive();
            },

            _quickClose: function(activity) {
                var quickClose = false;
                if (activity == -1) {
                    quickClose = true;
                }
                else {
                    var delta = (+new Date) - activity;
                    if (delta > 1000 * (options.idleAfter + options.warningLength)) {
                        quickClose = true;
                    }
                }
                if (quickClose) {
                    if (this.counter > 0) {
                        window.clearInterval(this.countdown);
                    }
                    options.onTimeout.call(warning);
                }
                return quickClose;
            },

            _idle: function() {
                var self = this, warning = this.warning[0];

                this.counter = options.warningLength;

                // fire the onIdle function
                options.onIdle.call(warning);

                // set inital value in the countdown placeholder
                options.onCountdown.call(warning, this.counter);

                // create a timer that runs every second
                this.countdown = window.setInterval(function() {
                    self.counter -= 1;

                    if (self.counter === 0) {
                        window.clearInterval(self.countdown);
                        options.onTimeout.call(warning);
                    }
                    else {
                        options.onCountdown.call(warning, self.counter);
                        self._keepAlive();
                    }
                }, 1000);
            },

            _startTimer: function() {
                this.timer = setTimeout(
                    $.proxy(this._keepAlive, this),
                    options.pollingInterval * 1000);
            },

            _stopTimer: function() {
                clearTimeout(this.timer);
            },

            _syncTimers: function(activity, saveActivity) {
                if (this._quickClose(activity)) {
                    return;
                }
                warning = this.warning[0];
                if (activity > saveActivity) {
                    if (this.countdownOpen) {
                        this.countdownOpen = false;
                        if (this.counter > 0) {
                            window.clearInterval(this.countdown);
                            this.counter = -1;
                            options.onResume.call(warning);
                        }
                    }
                    this.lastActivity = activity;
                    var delta = (+new Date) - activity;
                    $.idleTimer('destroy');
                    $.idleTimer(options.idleAfter * 1000 - delta);
                }
                if (this.countdownOpen) {
                    if (this.counter == -1) {
                        this._idle();
                    }
                }
                else {
                    this._startTimer();
                }
            },

            _keepAlive: function() {
                var self = this;
                var activity = this.lastActivity;
                var saveActivity = this.lastActivity;
                var done = 0;

                $.each(options.keepAliveURLs, function(i, url) {
                    // when countdown is open only ping SSM (first keepAliveURL)
                    if (!self.countdownOpen || i == 0) {
                        done++;
                    }
                });

                $.each(options.keepAliveURLs, function(i, url) {
                    if (!self.countdownOpen || i == 0) {
                        var target = url + options.keepAlivePage + "?activity=" + activity;
                        $.ajax({
                            url: target,
                            cache: false,
                            dataType: 'json',
                            success: function(data) {
                                var sessionActivity = Number(data.sessionActivity);
                                if (sessionActivity == -1) {
                                    // quickClose
                                    activity = -1;
                                }
                                else if (activity != -1 && sessionActivity > activity) {
                                    activity = sessionActivity;
                                }
                            },
                            complete: function() {
                                done--;
                                if (done == 0) {
                                    self._syncTimers(activity, saveActivity);
                                }
                            }
                        });
                    }
                });
            }
        };

        // run this thang
        IdleTimeout.init();
    };

    $.idleTimeout.options = {
        // number of seconds after user is idle to show the warning
        warningLength: 30,

        // url to call to keep the session alive while the user is active
        keepAliveURLs: [],

        // page to call to keep the session alive while the user is active
        keepAlivePage: "",

        // the response from keepAliveURL must equal this text:
        serverResponseEquals: "OK",

        // user is considered idle after this many seconds.  10 minutes default
        idleAfter: 600,

        // a polling request will be sent to the server every X seconds
        pollingInterval: 60,

        /*
            Callbacks
            "this" refers to the element found by the first selector passed to $.idleTimeout.
        */
        // callback to fire when the session times out
        onTimeout: function() {},

        // fires when the user becomes idle
        onIdle: function() {},

        // fires during each second of warningLength
        onCountdown: function() {},

        // fires when the user resumes the session
        onResume: function() {}
    };

})(jQuery);
